# -*- coding: utf-8 -*-

from Handlers import BaseHandler
import http.client
import json
import codecs
import os
import subprocess
from mako.template import Template


template_ppp = Template("""\
START=${START}
PROVIDER=${PROVIDER}
PERSIST=1
RESET_ON_FAIL=${RESET_ON_FAIL}
RESET_ALWAYS=${RESET_ALWAYS}
RESET_AT=${RESET_AT}
RESET_WAIT=${RESET_WAIT}
HOLDOFF=${HOLDOFF}

# PPP config
APN=${APN}
PDPTYPE=${PDPTYPE}
DEMAND=${DEMAND}
IDLE=${IDLE}
DEVICE=${DEVICE}
LOCALADDR=${LOCALADDR}
REMOTEADDR=${REMOTEADDR}
NETMASK=
DEFAULTROUTE=${DEFAULTROUTE}
AUTH=${AUTH}
ACCOUNT=${ACCOUNT}
PASSWORD=${PASSWORD}
NETMODE=${NETMODE}

#START=yes|no
#PROVIDER=
#PERSIST=1|0
#RESET_ON_FAIL=1|0
#RESET_ALWAYS=1|0
#RESET_AT=03:00
#RESET_WAIT=30
#HOLDOFF=5

#APN=example.jp
#PDPTYPE=IP|PPP
#DEMAND=1|0
#IDLE=30
#DEVICE=ttyLISA0|ttyKYM11|ttyUM03KO0|ttyAMP0|ttyUSBx|ttyACMx| \
#       LISA-U200|KYM11|UM03-KO|AMP520|AUTO
#LOCALADDR=
#REMOTEADDR=
#NETMASK=
#DEFAULTROUTE=1|0
#AUTH=CHAP|PAP
#ACCOUNT=account
#PASSWORD=password
#NETMODE=AUTO|WCDMA

""")


template_crg = Template("""\
ENABLE=${ENABLE}
WAKEUP=1
RESET_ON_FAIL=${RESET_ON_FAIL}
RESET_ALWAYS=${RESET_ALWAYS}
RESET_AT=${RESET_AT}
RESET_WAIT=${RESET_WAIT}
HOLDOFF=${HOLDOFF}

# Domain No. 0
DOMAIN0=${DOMAIN0}
DEMAND0=${DEMAND0}
LOCALADDR0=${LOCALADDR0}
REMOTEADDR0=${REMOTEADDR0}
DEFAULTROUTE0=${DEFAULTROUTE0}
STATICROUTES0=${STATICROUTES0}
IDLE0=${IDLE0}
ID0=${ID0}
PASSWORD0=${PASSWORD0}

# --- Example ---
# ENABLE=1
# WAKEUP=0
#
# RESET_ON_FAIL=1|0
# RESET_ALWAYS=1|0
# RESET_AT=03:00
# RESET_WAIT=30
# HOLDOFF=5

# DOMAINx=host.example.jp
# DEMANDx=1|0
# LOCALADDRx=192.168.5.34
# REMOTEADDRx=
# DEFAULTROUTEx=
# STATICROUTESx=10.254.0.0/24,10.253.0.0/24
# IDLEx=30
# IDx=userid
# PASSWORDx=password

""")


#-------------------------------------------------------------------------------
#
#-------------------------------------------------------------------------------
class Module(BaseHandler.BaseHandler):
	#=======================================================
	# 
	#=======================================================
	def __init__(self):
		self.ppp_config = "/etc/default/ppp"
		self.crg_config = "/etc/default/crg"
		self.name = 'PPP'
		self.ppp_keys = ['START','PROVIDER','RESET_ON_FAIL','RESET_ALWAYS','RESET_AT','RESET_WAIT','HOLDOFF',
					 'APN','PDPTYPE','DEMAND','IDLE','DEVICE','LOCALADDR','REMOTEADDR','DEFAULTROUTE','AUTH','ACCOUNT','PASSWORD','NETMODE']
		self.crg_keys = ['ENABLE','RESET_ON_FAIL','RESET_ALWAYS','RESET_AT','RESET_WAIT','HOLDOFF']
		self.domain_keys = ['DOMAIN','DEMAND','LOCALADDR','REMOTEADDR','DEFAULTROUTE','STATICROUTES','IDLE','ID','PASSWORD']

		self.upper = ['PDPTYPE','AUTH','NETMODE']
		self.mw_conf = "/etc/default/mobile_watch"
		self.crg_num = 1
		if os.path.exists('/dev/ttyKYM11'):
			self.crg_flg = True
		else:
			self.crg_flg = False
	#=======================================================
	# 
	#=======================================================
	def Get(self, request):

		status = 200
		type = request.query.type
		if type == 'config':
			result = {}
			result['PPP'] = self.readPPPConfig()
			if self.crg_flg:
				result['CRG'] = self.readCRGConfig()

		elif type == 'status':
			result = self.getStatus()

		#print(result)
		return result, status

	#=======================================================
	# 
	#=======================================================
	def Put(self, request):
		
		status = 400
		#print(request.json)
		result = self.writePPPConfig(request.json['PPP'])
		if result:
			if self.crg_flg:
				result = self.writeCRGConfig(request.json['CRG'])
				if result:
					self.restart('crg')
					self.restart('crgin')
			if result:
				self.restart('ppp')
				status = 200

		return status
		
	#=======================================================
	# 
	#=======================================================
	def Post(self, request):
	
		status = 405
		#print(request.json)
		if request.json['operation'] == 'ota':
			status = self.execOTA()

		return status

	#=======================================================
	# 
	#=======================================================
	def readPPPConfig(self):
		
		conf = {}

		for key in self.ppp_keys:
			if key == 'RESET_AT':
				conf['RESET_AT_ENABLE'] = 'no'
				conf['RESET_AT_TIME'] = ''
			elif key == 'CONNECTTYPE':
				conf['CONNECTTYPE'] = 'persist'
			else:
				conf[key] = ''

		config = open(self.ppp_config, "r")
		for line in config:
			if line.startswith('#') == True:
				pass
			else:
				sline = line.strip('\n').split('=')
				if len(sline) > 1 and sline[1]:
					if sline[0] in self.ppp_keys:
						if sline[0] in self.upper:
							conf[sline[0]] = sline[1].lower()
						elif sline[0] == 'RESET_AT':
							conf['RESET_AT_ENABLE'] = 'yes'
							conf['RESET_AT_TIME']  = ''.join((sline[1][:2],sline[1][3:]))
						else:
							conf[sline[0]] = sline[1]

		return conf

	#=======================================================
	# 
	#=======================================================
	def writePPPConfig(self, args):

		conf = {}
		status = True
		for key in self.ppp_keys:
			value = ''
			if key == 'DEVICE':
				value  = 'AUTO'
			elif key == 'RESET_AT':
				if 'RESET_AT_ENABLE' in args and args['RESET_AT_ENABLE']=='yes' and 'RESET_AT_TIME' in args and args['RESET_AT_TIME']:
					value = ':'.join((args['RESET_AT_TIME'][:2],args['RESET_AT_TIME'][2:]))
				else:
					value = ''
			elif key in self.upper:
				value = args[key].upper()
			else:
				value = args[key]
			conf[key] = value

		try:
			config = template_ppp.render(**conf)
			fd = open(self.ppp_config, 'w')
			fd.write(config)
			fd.close()
		except:
			status = False
		finally:
			return status

	#=======================================================
	# 
	#=======================================================
	def getStatus(self):
		
		mobile_status = {'level':'', 'carrier':'','ipaddress':'','modulename':'', 'tel_no':'' }
		mw_service = self.service_status('mobile_watch')

		if 'running' in mw_service:

			fd = open(self.mw_conf, 'r')
			for line in fd:
				sline = line.strip('\n').split('=')
				if sline[0] == 'HOST':
					host = sline[1]
				elif sline[0] == 'PORT':
					port = sline[1]

			reader = codecs.getreader("utf-8")
			try:
				conn = http.client.HTTPConnection(':'.join((host,port)), timeout=3)
				conn.request("GET", "/antenna")
				obj = json.load(reader(conn.getresponse()))
				
				if 'level' in obj:
					mobile_status['level'] = str(obj['level'])
				if 'carrier' in obj:
					mobile_status['carrier'] = str(obj['carrier'])

				conn.request("GET", "/tel_no")
				obj = json.load(reader(conn.getresponse()))
				if 'tel_no' in obj:
					mobile_status['tel_no'] = str(obj['tel_no'])
					if mobile_status['tel_no'] == 'Unknown':
						mobile_status['tel_no'] = ''
			except:
				pass

			if mobile_status['carrier'] == '1':
				mobile_status['ipaddress'] = self.getAddr('ppp0')
			if self.crg_flg:
				mobile_status['ipaddress_crg0'] = self.getAddr('ppp500')

			if os.path.exists('/tmp/.lisa_found'):
				mobile_status['modulename'] = 'LISA'
			elif os.path.exists('/tmp/.kym11_found'):
				mobile_status['modulename'] = 'KYM11' 
			elif os.path.exists('/tmp/.um03ko_found'):
				mobile_status['modulename'] = 'UM03KO'
			elif os.path.exists('/tmp/.amp520_found'):
				mobile_status['modulename'] = 'AMP520'

		return mobile_status
	#=======================================================
	# 
	#=======================================================
	def restart(self, service):

		self.service(service, 'restart')

	#=======================================================
	# 
	#=======================================================
	def readCRGConfig(self):

		conf = {}
		for key in self.crg_keys:
			if key == 'RESET_AT':
				conf['RESET_AT_ENABLE'] = 'no'
				conf['RESET_AT_TIME']  =''
			else:
				conf[key] = ''

		for i in range(self.crg_num):
			conf['DOMAIN%d' % i] = dict(zip(self.domain_keys, ['','','','','','','','']))
		
		config = open(self.crg_config, "r")
		for line in config:
			if line.startswith('#') or line.startswith('\n') or line.startswith('WAKEUP'):
				pass
			else:
				sline = line.strip('\n').split('=')
				if len(sline) > 1:
					if sline[0] in self.crg_keys:
						if sline[0] == 'RESET_AT':
							if sline[1]:
								conf['RESET_AT_ENABLE'] = 'yes'
								conf['RESET_AT_TIME']  = ''.join((sline[1][:2],sline[1][3:]))
						else:
							conf[sline[0]] = sline[1]
					else:
						key = sline[0][:-1]
						domain = 'DOMAIN%s' % sline[0][-1]
						if domain in conf:
							if key in self.domain_keys:
								conf[domain][key] = sline[1]
		return conf

	#=======================================================
	# 
	#=======================================================
	def writeCRGConfig(self, args):
		
		status = True
		conf = {}
		conf['ENABLE'] = args['ENABLE']
		for key in self.crg_keys:
			if key == 'RESET_AT':
				if 'RESET_AT_ENABLE' in args and args['RESET_AT_ENABLE']=='yes' and \
					'RESET_AT_TIME' in args and args['RESET_AT_TIME']:
					conf[key] = ':'.join((args['RESET_AT_TIME'][:2],args['RESET_AT_TIME'][2:]))
				else:
					conf[key] = ''
			else:
				conf[key] = args[key]

		for i in range(self.crg_num):
			num = str(i)
			for key in self.domain_keys:
				conf[key+num] = args['DOMAIN'+num][key]
		try:
			config = template_crg.render(**conf)
			fd = open(self.crg_config, 'w')
			fd.write(config)
			fd.close()
		except:
			status = False
		finally:
			return status

	#=======================================================
	# 
	#=======================================================
	def execOTA(self):
		
		status = 400 

		result,output = subprocess.getstatusoutput('kym11util OTA')
		if result == 0:
			if output.find('Already') <= 0:
				status = 409 
			else:
				status = 200 

		return status
