# -*- coding: utf-8 -*-

from Handlers import BaseHandler

#-------------------------------------------------------------------------------
#
#-------------------------------------------------------------------------------
class Serial(BaseHandler.BaseHandler):
	#=======================================================
	# 
	#=======================================================
	def __init__(self):
		self.config = "/etc/default/socat"
		self.name = 'Socat'
		self.ports = ['ttyO1','ttyO0']
		self.keys = ['MODE','PROTO','LCLPORT','DSTADDR','DSTPORT','BAUD','CHAR','STOP','PARITY','RTSCTS','TIME','MIN']
		self.upper = ['MODE', 'PROTO', 'PARITY']
	#=======================================================
	# 
	#=======================================================
	def Get(self, request):

		status = 200
		return self.readConfig(), status

	#=======================================================
	# 
	#=======================================================
	def Put(self, request):
		
		status = 200
		args = request.json
		if self.validator(args):
			self.writeConfig(args)
			self.restart(args)
		else:
			status= 409
		return status
		
	#=======================================================
	# 
	#=======================================================
	def readConfig(self):
		
		conf = {}
		for port in self.ports:
			conf[port] = {}
			conf[port]['USE'] = 'no'
			for key in self.keys:
				conf[port][key] = ''

		config = open(self.config, "r")
		for line in config:
			if line.startswith('tty') == True:
				sline = line.strip().split('=')
				if sline[0] in self.ports:
					conf[sline[0]]['USE'] = sline[1]
				else:
					port, key = sline[0].split('_')
					conf[port][key] = sline[1].strip('\n').lower()

		return conf

	#=======================================================
	# 
	#=======================================================
	def writeConfig(self, args):

		config = open(self.config, 'w')

		for port in self.ports:
			config.write('='.join((port, args[port]['USE']))+ '\n')
			for key in self.keys:
				value = ''
				if key in args[port]:
					value = args[port][key]
				if key in self.upper:
					value = value.upper()
				value = value + '\n'
				config.write('='.join(('_'.join((port, key)), value)))

	#=======================================================
	# 
	#=======================================================
	def validator(self, args):
		
		result = True
		used_port = self.used_port()
		for port in self.ports:
			if args[port]['USE'] == 'yes':
				process =  self.port_process(args[port]['PROTO'], args[port]['LCLPORT'])
				if process and not 'socat' in process:
					if args[port]['PROTO'] == 'tcp':
						if args[port]['LCLPORT'] in used_port['tcp'] or args[port]['LCLPORT'] in used_port['tcp6']:
							result = False
							break
					else:
						if args[port]['LCLPORT'] in used_port['udp'] or args[port]['LCLPORT'] in used_port['udp6']:
							result = False
							break

		return result

	#=======================================================
	# 
	#=======================================================
	def restart(self, args):
		for port in self.ports:
			service_name = "socat-%s" % port
			self.service(service_name, 'restart')
			"""
			if args[port]['USE'] == 'yes':
				self.service(service_name, 'restart')
			else:
				self.service(service_name, 'stop')
			"""
